<?php
require_once('watermark.inc.php');

require_once('../include/photo/synophoto_csPhotoAlbum.php');
require_once('../include/photo/synophoto_csPhotoMisc.php');
require_once SYNOPHOTO_INCLUDE_CACHE;

define('BIN_SYNOPHOTO_WATERMARK_UTIL', '/usr/syno/bin/synophoto_watermark_util');
define('BIN_SYNOPHOTO_DSM_USER', '/usr/syno/bin/synophoto_dsm_user');
define('SYNOPHOTO_DSM_USER_GET_SHARE', '--getshare');
define('SYNOPHOTO_DSM_USER_LIST_SHARE', '--listshare');
define('ID_PREFIX', 'dsmshare_');

class WatermarkUtil {
	private static $settingsCache = null;

	/**
	 * Gets the name from the specified ID of a shared folder
	 */
	public static function getShareNameById($id) {
		$ids = explode('_', $id);
		return (((count($ids) !== 2) || ($ids[0] !== 'album')) ? false : pack('H*', $ids[1]));
	}

	/**
	 * Gets the pixel dimensions of the input image path
	 *
	 * @param $img The input image path
	 * @return An array with at least two elements: ( '0' => width, '1' => height [, '2' => type, '3' => dimension string, 'mime' => mime type, 'channels' => 3 for RGB and 4 for CYMK images, 'bits' => number of bits for each color] )
	 */
	public static function getImageSize($img) {
		//if (!$img || !file_exists($img)) {
		//	return array(0, 0);
		//}

		// Basically using the PHP getimagesize function
		$imgsize = @getimagesize($img);

		// Sometimes it may fail, mostly due to privilege issues
		if (!$imgsize) {
			$output = '';
			csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_WATERMARK_UTIL, array('-e', $img), false, $output);
			$_imgsize = explode(' ', $output[0]);
			$imgsize = array((int)$_imgsize[0], (int)$_imgsize[1]);
		}
		return $imgsize;
	}

	/**
	 * Gets watermark settings from database
	 *
	 * @param $shareName The name of the shared folder
	 */
	public static function getSettingsByShareName($shareName) {
		if (empty($shareName)) {
			return null;
		}

		$slash = strpos($shareName, '/');
		if ($slash !== false) {
			$shareName = substr($shareName, 0, $slash);
		}

		if (!self::$settingsCache) {
			self::$settingsCache = new Cache();
		}
		if (self::$settingsCache->check($shareName)) {
			return self::$settingsCache->get($shareName);
		}

		$query = 'SELECT path, opacity, gravity, size, hash, mtime, changed FROM watermark WHERE shareid IN (SELECT shareid FROM photo_share WHERE sharename = ?)';
		$param = array($shareName);
		$statement = PHOTO_DB_Query($query, $param);
		$row = PHOTO_DB_FetchRow($statement);

		$value = null;
		if ($row) {
			$path = SYNOPHOTO_WATERMARK_DIR . '/' . $row['path'];
			$imgsize = self::getImageSize($path);

			$value = array(
				'id' => (ID_PREFIX . bin2hex($path)),
				'inherit' => ($slash !== false),
				'path' => $path,
				'opacity' => (int)$row['opacity'],
				'gravity' => (int)$row['gravity'],
				'size'=> (int)$row['size'],
				'title' => pathinfo($path)['basename'],
				'width' => $imgsize[0],
				'height' => $imgsize[1],
				'hash' => $row['hash'],
				'mtime' => (int)$row['mtime'],
				'changed' => (bool)$row['changed']
			);
		}

		self::$settingsCache->set($shareName, $value);
		return $value;
	}

	/**
	 * Gets watermark settings from database
	 *
	 * @param $id The ID of the shared folder
	 */
	public static function getSettingsById($id) {
		$shareName = self::getShareNameById($id);
		return ($shareName ? self::getSettingsByShareName($shareName) : null);
	}

	/**
	 * Saves watermark settings to database
	 *
	 * @param $shareName The name of the shared folder
	 */
	public static function saveSettingsByShareName($shareName, $ori_path, $opacity, $gravity, $size) {
		$query = '';
		$params = array();
		$path = pathinfo($ori_path, PATHINFO_BASENAME);
		$changed = self::haveSettingsChanged($shareName, $path, $opacity, $gravity, $size);
		if ($changed === null) {
			$changed = true;
			$query = "SELECT shareid FROM photo_share WHERE sharename = ?";
			$params = array($shareName);
			$statement = PHOTO_DB_Query($query, $params);
			$id = PHOTO_DB_FetchRow($statement)['shareid'];

			$query = "INSERT INTO watermark (shareid, path, opacity, gravity, size, hash, mtime) VALUES (?, ?, ?, ?, ?, ?, ?)";
			$params = array($id, $path, $opacity, $gravity, $size, (file_exists($ori_path) ? hash_file('crc32b', $ori_path) : ''), time());
		} elseif ($changed) {
			$query = "UPDATE watermark SET path = ?, opacity = ?, gravity = ?, size = ?, hash = ?, mtime = ?, changed = 't' WHERE shareid IN (SELECT shareid FROM photo_share WHERE sharename = ?)";
			$params = array($path, $opacity, $gravity, $size, (file_exists($ori_path) ? hash_file('crc32b', $ori_path) : ''), time(), $shareName);
		} else {
			$query = "UPDATE watermark SET changed = 'f' WHERE shareid IN (SELECT shareid FROM photo_share WHERE sharename = ?)";
			$params = array($shareName);
		}
		PHOTO_DB_Query($query, $params);

		if (self::$settingsCache) {
			self::$settingsCache->delete($shareName);
		}

		return $changed;
	}

	/**
	 * Saves watermark settings to database
	 *
	 * @param $id The ID of the shared folder
	 */
	public static function saveSettingsById($id, $path, $opacity, $gravity, $size) {
		$shareName = self::getShareNameById($id);
		return ($shareName && self::saveSettingsByShareName($shareName, $path, $opacity, $gravity, $size));
	}

	public static function clearSettingsByShareName($shareName) {
		$query = 'DELETE FROM watermark WHERE shareid IN (SELECT shareid FROM photo_share WHERE sharename = ?)';
		$params = array($shareName);
		PHOTO_DB_Query($query, $params);

		if (self::$settingsCache) {
			self::$settingsCache->delete($shareName);
		}
		return true;
	}

	public static function clearSettingsById($id) {
		$shareName = self::getShareNameById($id);
		return ($shareName && self::clearSettingsByShareName($shareName));
	}

	/**
	 * Gets pixel dimensions of the cover of the specified album by share name
	 */
	public static function getAlbumCoverInfoByShareName($shareName) {
		$cover_info = array(
			//'info' => array(),
			'w' => 0,
			'h' => 0,
			'hash' => '');
		//$info = csSYNOPhotoAlbum::GetAlbumInstance()->GetAlbumCover($shareName);
		$path = csSYNOPhotoAlbum::GetAlbumInstance()->GetAlbumCover($shareName)['coverPath'];
		$realpath = $path;
		if (SYNOPHOTO_ADMIN_USER !== 'root') { // PPS
			$realpath = csSYNOPhotoMisc::GetUserRealHomePath(SYNOPHOTO_ADMIN_NAME) . '/photo/' . $path;
		}
		if (file_exists($realpath)) {
			$size = AlbumAPIUtil::GetCoverResolutionRotation($path);
			//$cover_info['info'] = $info;
			$cover_info['w'] = ($size[2] ? $size[1] : $size[0]);
			$cover_info['h'] = ($size[2] ? $size[0] : $size[1]);
			$cover_info['hash'] = hash_file('crc32b', $realpath);
		}
		return $cover_info;
	}

	/**
	 * Gets pixel dimensions of the cover of the specified album by ID
	 */
	public static function getAlbumCoverInfoById($id) {
		$shareName = self::getShareNameById($id);
		return ($shareName ? self::getAlbumCoverInfoByShareName($shareName) : array());
	}

	/**
	 * Get the currently logged in user
	 *
	 * @sa dsm_share.php
	 */
	public static function getUser() {
		$user = 'guest';
		if ($_SESSION[SYNOPHOTO_ADMIN_USER]['use_dsm_account']) {
			if (isset($_SESSION[SYNOPHOTO_ADMIN_USER]['reg_syno_user'])) {
				$user = $_SESSION[SYNOPHOTO_ADMIN_USER]['reg_syno_user'];
			} elseif (isset($_SESSION[SYNOPHOTO_ADMIN_USER]['admin_syno_user'])) {
				$user = ('root' === SYNOPHOTO_ADMIN_USER) ? 'admin' : SYNOPHOTO_ADMIN_NAME;
			}
		} else {
			if (isset($_SESSION[SYNOPHOTO_ADMIN_USER]['reg_syno_user'])) {
				$user = 'PhotoStation';
			} elseif (isset($_SESSION[SYNOPHOTO_ADMIN_USER]['admin_syno_user'])) {
				$user = ('root' === SYNOPHOTO_ADMIN_USER) ? 'admin' : SYNOPHOTO_ADMIN_NAME;
			}
		}
		return $user;
	}

	/**
	 * Lists accessible folders
	 */
	public static function getShare($root) {
		$children = array();
		$root = explode('_', $root)[1];
		if ($root === '') {
			// Get user accessible public shares, using /usr/syno/bin/synophoto_dsm_user --getshare
			$user = self::getUser();
			$output = '';
			//$cmd = implode(' ', array(
			//	BIN_SYNOPHOTO_DSM_USER,
			//	SYNOPHOTO_DSM_USER_GET_SHARE,
			//	escapeshellarg($user)));
			//exec($cmd, $output);
			csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_DSM_USER, array(SYNOPHOTO_DSM_USER_GET_SHARE, $user), false, $output);
			foreach ($output as $line) {
				$data = explode("\t", $line);
				if (2 > count($data)) {
					continue;
				}
				$text = trim($data[0]);
				$id = trim($data[1]);
				if ($text === 'homes') {
					// Only show the user home directory
					$text = 'home';
					//$id = $id . '/' . $user;
					$id = csSYNOPhotoMisc::GetUserRealHomePath($user);
				} elseif ($text === 'home') {
					continue;
				}
				$children[] = array(
					'id' => ID_PREFIX . bin2hex($id),
					'text' => $text
					//'children' => self::getShare($id)
				);
			}
		} else {
			// List subdirs, using /usr/syno/bin/synophoto_dsm_user --listshare
			$root = pack('H*', $root);
			$output = '';
			//$cmd = implode(' ', array(
			//	BIN_SYNOPHOTO_DSM_USER,
			//	SYNOPHOTO_DSM_USER_LIST_SHARE,
			//	escapeshellarg(WatermarkUtil::getUser()),
			//	escapeshellarg($root),
			//	'dir'));
			//exec($cmd, $output);
			csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_DSM_USER, array(SYNOPHOTO_DSM_USER_LIST_SHARE, self::getUser(), $root, 'dir'), false, $output);
			foreach ($output as $line) {
				if (csSYNOPhotoMisc::IsSkipDir($line)) {
					continue;
				}
				$path = $root . '/' . $line;
				$children[] = array(
					'id' => ID_PREFIX . bin2hex($path),
					'text' => $line
					//'children' => self::getShare($path)
				);
			}
		}

		// Sort the results by display names
		usort($children, function ($lhs, $rhs) {
			return strnatcasecmp($lhs['text'], $rhs['text']);
		});

		return $children;
	}

	/**
	 * Creates user watermark directory (if not exists) and returns its path
	 *
	 * @return The path \e WATERMARK_DIR with user appended
	 */
	public static function getUserWatermarkDir() {
		$arg = array('-m');

		if (!file_exists(SYNOPHOTO_WATERMARK_DIR)) {
			if (SYNOPHOTO_IS_PPS) {
				$arg[] = '-p';
				$arg[] = self::getUser();
			}
			csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_WATERMARK_UTIL, $arg);
		}
		return SYNOPHOTO_WATERMARK_DIR;
	}

	/**
	 * Lists image files in the specified directory
	 */
	public static function listShare($id) {
		do {
			if (!$id) {
				break;
			}
			$info = explode('_', $id);
			$dir = pack('H*', $info[1]);

			// Using user watermark directory instead of standard DSM shares
			if ($info[0] === 'watermark') {
				$dir = self::getUserWatermarkDir();
			}

			$output = '';
			//$cmd = implode(' ', array(
			//	BIN_SYNOPHOTO_DSM_USER,
			//	SYNOPHOTO_DSM_USER_LIST_SHARE,
			//	escapeshellarg(self::getUser()),
			//	escapeshellarg($dir),
			//	'file'));
			//exec($cmd, $output);
			csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_DSM_USER, array(SYNOPHOTO_DSM_USER_LIST_SHARE, self::getUser(), $dir, 'file'), false, $output);
			$resp = array('items' => array());
			foreach ($output as $line) {
				// Only image files (without gif) are cared about
				if (!csSYNOPhotoMisc::IsPhotoFile($line) || self::isIgnored($line)) {
					continue;
				}

				$path = $dir . '/' . $line;
				$imgsize = self::getImageSize($path);
				$resp['items'][] = array(
					'id' => ID_PREFIX . bin2hex($path),
					'title' => $line,
					'width' => $imgsize[0],
					'height' => $imgsize[1],
					'path' => $path
					//'imgsize' => $imgsize
				);
			}

			// Sort the results by display names
			usort($resp['items'], function ($lhs, $rhs) {
				return strnatcasecmp($lhs['title'], $rhs['title']);
			});
			return $resp;
		} while (false);
		return false;
	}

	/**
	 * Performs \e cp or \e mv, determined by \p $keep
	 *
	 * @param $keep The option that determines if the source file should be kept
	 */
	public static function copy($src, $file) {
		$arg = array('-c', $src, '-d', $file, '-u', self::getUser());
		if (SYNOPHOTO_IS_PPS) {
			$arg[] = '-p';
			$arg[] = self::getUser();
		}
		csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_WATERMARK_UTIL, $arg);
	}

	/**
	 * Performs chown, restricted to the currently logged in user
	 */
	public static function chown($file) {
		$arg = array('-o', $file, '-u', self::getUser());
		if (SYNOPHOTO_IS_PPS) {
			$arg[] = '-p';
			$arg[] = self::getUser();
		}
		csSYNOPhotoMisc::ExecCmd(BIN_SYNOPHOTO_WATERMARK_UTIL, $arg);
	}

	/**
	 * Tests if this string starts with the specified prefix
	 *
	 * @param $haystack The string to be tested
	 * @param $needle The specified prefix
	 * @return If the character sequence represented by the argument \p $needle is a prefix of the character sequence represented by the argument \p $haystack
	 */
	public static function startsWith($haystack, $needle) {
		return ($needle === "") || (strrpos($haystack, $needle, -strlen($haystack)) !== FALSE);
	}

	/**
	 * Copies the selected watermark to user watermark directory
	 */
	public static function copyWatermark($src, $file) {
		self::copy($src, $file);

		// Change owner for DSM to count disk usage correctly
		self::chown($file);
	}

	/**
	 * Checks if watermark settings have changed
	 */
	public static function haveSettingsChanged($shareName, $path, $opacity, $gravity, $size) {
		$settings = self::getSettingsByShareName($shareName);
		if (!$settings) {
			return null;
		} else {
			return (($settings['path'] !== $path)
				|| ($settings['opacity'] !== $opacity)
				|| ($settings['gravity'] !== $gravity)
				|| ($settings['size'] !== $size));
		}
	}

	public static function blendPhoto($album, $photo) {
		$arg = '';
		if (SYNOPHOTO_ADMIN_USER !== 'root') {
			// Personal Photo Station
			$arg = 'u:' . SYNOPHOTO_ADMIN_NAME . '/' . $album . '/' . $photo;
		} else {
			// Photo Station
			$arg = 'w:' . $album . '/' . $photo;
		}
		//$cmd = implode(' ', array(
		//	SYNOPHOTO_DAEMON,
		//	'-a',
		//	escapeshellarg($arg)));
		//exec($cmd);
		csSYNOPhotoMisc::ExecCmd(SYNOPHOTO_DAEMON, array('-a', $arg));
	}

	public static function blendAlbum($album) {
		$arg = '';
		if (SYNOPHOTO_ADMIN_USER !== 'root') {
			// Personal Photo Station
			$arg = 'U:' . SYNOPHOTO_ADMIN_NAME . '/' . $album;
		} else {
			// Photo Station
			$arg = 'W:' . $album;
		}
		//$cmd = implode(' ', array(
		//	SYNOPHOTO_DAEMON,
		//	'-a',
		//	escapeshellarg($arg)));
		//exec($cmd);
		csSYNOPhotoMisc::ExecCmd(SYNOPHOTO_DAEMON, array('-a', $arg));
	}

	public static function removeAlbum($album) {
		$arg = '';
		if (SYNOPHOTO_ADMIN_USER !== 'root') {
			// Personal Photo Station
			$arg = 'r:' . SYNOPHOTO_ADMIN_NAME . '/' . $album;
		} else {
			// Photo Station
			$arg = 'R:' . $album;
		}
		//$cmd = implode(' ', array(
		//	SYNOPHOTO_DAEMON,
		//	'-a',
		//	escapeshellarg($arg)));
		//exec($cmd);
		csSYNOPhotoMisc::ExecCmd(SYNOPHOTO_DAEMON, array('-a', $arg));
	}

	public static function isIgnored($path) {
		return (strcasecmp(pathinfo($path)['extension'], 'gif') === 0);
	}
} // class WatermarkUtil
?>
