<?php

/**
 * copy people tag in Photo Station DB to file metadata if the tag is not exist in file metadata
 */

namespace merge_ms_people_tag;

const SYNO_PHOTO_MERGED_KEY = 'ms_people_tag_merged',
	SYNO_PHOTO_EXIV2_PROG = '/usr/bin/exiv2';

function run()
{
	global $params, $needReIndex;

	if ('yes' === $params[SYNO_PHOTO_MERGED_KEY]) {
		//key "ms_people_tag_merged" in setting.conf is set
		setMergedKeyToDB(SYNO_PHOTO_MERGED_KEY);
		return;
	}

	if (isSetMergedKeyInDB(SYNO_PHOTO_MERGED_KEY)) {
		//key "ms_people_tag_merged" in database is set
		return;
	}

	$photoList = getPhotoListFromDB();
	$metadataTagCount = 0;

	foreach ($photoList as $photo) {
		//read tags from db
		$dbList = readTagListFromDB($photo);

		//read tags from metadata
		$metadataList = readTagListFromMetadata($photo['path'], $metadataTagCount);

		//rename db tag if the metadata tag with the same name exist
		renameDuplicateTagInDBList($dbList, $metadataList);

		//write tags
		writeTagListToMetadata($photo, $dbList, $metadataTagCount);
	}

	setMergedKeyToDB(SYNO_PHOTO_MERGED_KEY);

	$needReIndex = true;
}

function getPhotoListFromDB()
{
	global $dbh;

	$photoList = array();

	$sqlCond = 'SELECT id, path, resolutionx, resolutiony FROM photo_image WHERE id IN (SELECT DISTINCT image_id FROM photo_image_label)';
	$sqlParam = array();

	$dbResult = $dbh->prepare($sqlCond);
	$dbResult->execute($sqlParam);

	while (false !== ($row = $dbResult->fetch())) {
		if (!inAcceptableExtList($row['path'])) {
			continue;
		}

		$photoList[] = array(
			'id' => $row['id'],
			'path' => $row['path'],
			'resolutionx' => $row['resolutionx'],
			'resolutiony' => $row['resolutiony']
		);
	}

	return $photoList;
}

function inAcceptableExtList($path)
{
	$acceptableExtList = array("jpg", "jpeg", "jpe", "png", "tiff", "tif", "cr2", "nef", "pef", "orf", "dng");

	foreach ($acceptableExtList as $acceptableExt) {
		if (0 === substr_compare(strtolower($path), $acceptableExt, strlen($path) - strlen($acceptableExt), strlen($acceptableExt))) {
			return true;
		}
	}

	return false;
}

function readTagListFromDB($photo)
{
	global $dbh;

	$tagList = array();

	$sqlCond =
		'SELECT photo_label.id, photo_label.name, photo_image_label.info FROM photo_image_label '.
		'INNER JOIN photo_label ON photo_image_label.label_id = photo_label.id '.
		"WHERE photo_label.category = 0 AND photo_image_label.info <> '' AND photo_image_label.info IS NOT NULL " .
		"AND photo_image_label.status = 't' AND photo_image_label.image_id = {$photo['id']}";
	$sqlParam = array();

	$dbResult = $dbh->prepare($sqlCond);
	$dbResult->execute($sqlParam);

	while (false !== ($row = $dbResult->fetch())) {
		$tagList[$row['name']] = array(
			'newName' => $row['name'],
			'infoFloat' => convertDBInfoToFloat($photo['resolutionx'], $photo['resolutiony'], json_decode($row['info'], true)),
			'isDuplicate' => false
		);
	}

	return $tagList;
}

function readTagListFromMetadata($path, &$metadataTagCount)
{
	global $userHome;

	$tmpTagList = array();
	$tagList = array();
	$peopleTagPrefix = 'Xmp.MP.RegionInfo/MPRI:Regions[';
	$tagIndex = 1;

	$filePath = $path;
	if (isPPS()) {
		$filePath = "$userHome/photo/$filePath";
	}

	//$cmd = SYNO_PHOTO_EXIV2_PROG.' -Pkt '.escapeshellarg($filePath);
	//@exec($cmd, $cmdResult);
	$cmdResult = '';
	csSYNOPhotoMisc::ExecCmd(SYNO_PHOTO_EXIV2_PROG, array('-Pkt', $filePath), false, $cmdResult);

	foreach ($cmdResult as $tag) {
		if ($peopleTagPrefix !== substr($tag, 0, strlen($peopleTagPrefix))) {
			continue;
		}

		$targetTagPrefix = $peopleTagPrefix.$tagIndex.']';
		$nextTagPrefix = $peopleTagPrefix.($tagIndex + 1).']';

		if ($nextTagPrefix === substr($tag, 0, strlen($nextTagPrefix))) {
			//already arrive next people tag
			$tagIndex++;
			continue;
		}

		$subTag = explode(' ', $tag);
		$key = $subTag[0];
		$value = trim(substr($tag, strlen($key), strlen($tag) - strlen($key)));

		if ($key === $targetTagPrefix.'/MPReg:PersonDisplayName') {
			$tmpTagList[$tagIndex]['PersonDisplayName'] = $value;
			$nameList[$value] = $value;
		} else if ($key === $targetTagPrefix.'/MPReg:Rectangle') {
			$tmpTagList[$tagIndex]['Rectangle'] = $value;
		}
	}

	$metadataTagCount = 0;
	foreach($tmpTagList as $tag) {
		if (!$tag['Rectangle']) {
			continue;
		}

		$metadataTagCount++;

		if (!$tag['PersonDisplayName']) {
			continue;
		}

		$tagList[$tag['PersonDisplayName']] = array(
			'Rectangle' => $tag['Rectangle']
		);
	}

	return $tagList;
}

function renameDuplicateTagInDBList(&$dbList, $metadataList)
{
	foreach ($dbList as $name => &$tag) {
		//check for duplicate
		if (isset($metadataList[$name]) && isInToleranceRange($tag['infoFloat'], $metadataList[$name]['Rectangle'])) {
			$tag['isDuplicate'] = true;
			continue;
		}

		$suffix = 1;
		$newName = $name;

		//trying to rename
		while (true) {
			if (isset($metadataList[$newName])) {
				$newName = $name.'_'.($suffix++);
				continue;
			}

			$arrayKeys = array_keys($dbList, $newName);
			$arrayKeysCount = count($arrayKeys);

			if (!$arrayKeysCount || (1 == $arrayKeysCount && $key === $arrayKeys[0])) {
				//no same tag name in current dbNameList || only one same tag name and it is this tag name itself
				break;
			}

			$newName = $name.'_'.($suffix++);
		}

		$tag['newName'] = $newName;
	}
}

function writeTagListToMetadata($photo, $dbList, $metadataTagCount)
{
	$count = $metadataTagCount;

	//insert new tags from DB into metadata
	foreach ($dbList as $name => $tag) {
		if (!strlen($name) || $tag['isDuplicate']) {
			continue;
		}

		insertTagIntoMetadata($photo, ++$count, $tag['newName'], convertDBInfoToString($tag['infoFloat']));
	}
}

function insertTagIntoMetadata($photo, $index, $newName, $rectangle)
{
	global $userHome;

	//$nameSpaceCmd = '-M"reg MP http://ns.microsoft.com/photo/1.2/" '.
	//	'-M"reg MPRI http://ns.microsoft.com/photo/1.2/t/RegionInfo#" '.
	//	'-M"reg MPReg http://ns.microsoft.com/photo/1.2/t/Region#"';
	$nameSpaceCmd = array(
		'-M', 'reg MP http://ns.microsoft.com/photo/1.2/',
		'-M', 'reg MPRI http://ns.microsoft.com/photo/1.2/t/RegionInfo#',
		'-M', 'reg MPReg http://ns.microsoft.com/photo/1.2/t/Region#');

	$filePath = $photo['path'];
	if (isPPS()) {
		$filePath = "$userHome/photo/$filePath";
	}

	if (1 === $index) {
		//$cmd = sprintf('%s %s %s %s', SYNO_PHOTO_EXIV2_PROG, $nameSpaceCmd,
		//			   '-M"set Xmp.MP.RegionInfo/MPRI:Regions XmpText type=Bag"',
		//			   escapeshellarg($filePath));
		//@exec($cmd);
		csSYNOPhotoMisc::ExecCmd(SYNO_PHOTO_EXIV2_PROG, array_merge($nameSpaceCmd, array(
			'-M', 'set Xmp.MP.RegionInfo/MPRI:Regions XmpText type=Bag',
			$filePath)));
	}

	//$cmd = sprintf('%s %s %s %s %s', SYNO_PHOTO_EXIV2_PROG, $nameSpaceCmd,
	//			   '-M"set Xmp.MP.RegionInfo/MPRI:Regions['.$index.']/MPReg:Rectangle '.$rectangle.'"',
	//			   '-M"set Xmp.MP.RegionInfo/MPRI:Regions['.$index.']/MPReg:PersonDisplayName '.str_replace('"', '\"', $newName).'"',
	//			   escapeshellarg($filePath));
	//@exec($cmd);
	csSYNOPhotoMisc::ExecCmd(SYNO_PHOTO_EXIV2_PROG, array_merge($nameSpaceCmd, array(
		'-M', 'set Xmp.MP.RegionInfo/MPRI:Regions[' . $index . ']/MPReg:Rectangle ' . $rectangle,
		'-M', 'set Xmp.MP.RegionInfo/MPRI:Regions[' . $index . ']/MPReg:PersonDisplayName ' . $newName,
		$filePath)));
}

function convertDBInfoToFloat($photox, $photoy, $info)
{
	$thumbSize = 800;

	$ratio = floatval($photoy) / floatval($photox);

	if ($ratio > 1) {//higher image
		$x = floatval($info['x']) / ($thumbSize / $ratio);
		$y = floatval($info['y']) / $thumbSize;
		$width = floatval($info['width']) / ($thumbSize / $ratio);
		$height = floatval($info['height']) / $thumbSize;
	} else {//wider image
		$x = floatval($info['x']) / $thumbSize;
		$y = floatval($info['y']) / ($thumbSize * $ratio);
		$width = floatval($info['width']) / $thumbSize;
		$height = floatval($info['height']) / ($thumbSize * $ratio);
	}

	//prevent 0 for width & height
	$width = $width > 0 ? $width : 0.000001;
	$height = $height > 0 ? $height : 0.000001;

	return array(
		'x' => $x,
		'y' => $y,
		'width' => $width,
		'height' => $height
	);
}

function convertDBInfoToString($infoFloat)
{
	$precision = 6;

	$x = number_format($infoFloat['x'], $precision, '.', '');
	$y = number_format($infoFloat['y'], $precision, '.', '');
	$w = number_format($infoFloat['width'], $precision, '.', '');
	$h = number_format($infoFloat['height'], $precision, '.', '');

	return "$x, $y, $w, $h";
}

function isInToleranceRange($infoFloat, $rectangle)
{
	$dbTolerance = 0.01;

	$recInfo = explode(',', $rectangle);
	foreach($recInfo as &$item) {
		$item = floatval(trim($item));
	}

	$arr = array(
		$dbTolerance,
		abs($infoFloat['x'] - $recInfo[0]),
		abs($infoFloat['y'] - $recInfo[1]),
		abs($infoFloat['width'] - $recInfo[2]),
		abs($infoFloat['height'] - $recInfo[3])
	);

	return $dbTolerance === max($arr);
}

?>
